import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import musicalrobot
import os

from musicalrobot import edge_detection as ed
from musicalrobot import pixel_analysis as pa

def dict_pack (d_files, d_names, d_crop, d_inftemp, d_temp, d_plate):
    """
    Function is used within the auto_crop function to  crop using the inputs
    given by the user.

    Parameters:
    -----------
    d_files : dictonary
        contains the raw file in the requested folder

    d_names : dictonary
        Contains all the names of the files in the requested folder

    d_crop : dictonary
        Contains the cropped versions of the files in the folder

    d_inftemp : dictonary
        Contins all of the inflection points of the files in the folder

    Returns
    --------
    d_all : dictonary
        Nested dictionary of all of the needed dictonary

    """
    d_all = {'d_files' : d_files, 'd_names' : d_names, 'd_crop' : d_crop, 'd_inftemp': d_inftemp, 'd_temp' : d_temp, 'd_plate' : d_plate}

    return d_all

def dict_unpack (d_all):
    """
    Function is used within the auto_crop function to  crop using the inputs
    given by the user.

    Parameters:
    -----------
    d_all : dictonary
        Nested dictionary of all of the needed dictonary

    Returns
    --------
    d_files : dictonary
        contains the raw file in the requested folder

    d_names : dictonary
        Contains all the names of the files in the requested folder

    d_crop : dictonary
        Contains the cropped versions of the files in the folder

    d_inftemp : dictonary
        Contins all of the inflection points of the files in the folder

    """
    d_files = d_all['d_files']
    d_names = d_all['d_names']
    d_crop = d_all['d_crop']
    d_inftemp = d_all['d_inftemp']
    d_temp = d_all['d_temp']
    d_plate = d_all['d_plate']

    return d_files, d_names, d_crop, d_inftemp, d_temp, d_plate

def image_crop (tocrop, top, bottom, left, right):
    """
    Function is used within the auto_crop function to  crop using the inputs
    given by the user.

    Parameters:
    -----------
    tocrop : array
        The raw tiff file that is stored in a dictionary and pulled from each
        key using a wrapper. Acts as the base image for the auto_crop

    left : int
        Number of pixels taken off of the left side of the image

    right : int
        Number of pixels taken off of the right side of the image

    top : int
        Number of pixels taken off of the top side of the image

    bottom : int
        Number of pixels taken off of the bottom side of the image

    Returns
    --------
    crop : array
        The array of the tiff file with the requested columns/rows removed

    """
    crop = []

    #checking tocrop is an array and converting if its not
    if isinstance(tocrop, np.ndarray) == True:
        pass
    else:
        tocrop = np.asarray(tocrop)

    frames, height, width = tocrop.shape
    for frame in tocrop:
        crop.append(frame[0 + top: height - bottom, 0 + left: width - right])

    return crop

def plot_image (crop, plotname):
    """
    Plots the given cropped image - used as an internal function

    Parameters:
    -----------
    crop: array
        The array of the tiff file with the requested columns/rows removed

    plotname : string
        Name pulled from the orginal file name - is the chart title

    Returns:
    --------
    No returns : will print the plot

    """

    plt.imshow(crop[50])
    plt.colorbar()
    plt.title(plotname)
    plt.show()

    return

def plot_profiles (temp, plate_temp, save_location):
    """

    """

    for i in range(len(temp[:96])):
        fig, ax = plt.subplots(1,2,figsize=(15,5))
        # Plotting frame number vs sample temp
        frame_number = np.linspace(1,len(temp[i]),len(temp[i]))
        #plot number 1
        ax[0].scatter(frame_number, temp[i], s=0.5)
        ax[0].set_title('Frame number vs Sample temp:Sample '+str(i+1))
        ax[0].set_xlabel('Frame_number')
        ax[0].set_ylabel('Sample temperature($^{\circ}$C)')

        # Plotting plate temp vs sample temp
        ax[1].scatter(plate_temp[i], temp[i], s=0.5)
        ax[1].set_title('Plate temp vs Sample temp:Sample '+str(i+1))
        ax[1].set_xlabel('Plate temperature($^{\circ}$C)')
        ax[1].set_ylabel('Sample temperature($^{\circ}$C)')
        
        plt.savefig(save_location + "/Sample " + str(i+1) + ".jpg")

    return


def choose_crop (tocrop, plotname):
    """
    Will ask user to choose if the image will be cropped or not. Will skip the
        specific image

    Allowed inputs are y or n. Any other inputs will result in a re-request

    Parameters:
    -----------
    crop: array
        The array of the tiff file with the requested columns/rows removed

    plotname : string
        Name pulled from the orginal file name - is the chart title

    Returns:
    --------
    crop: array
        The array of the tiff file with the requested columns/rows removed. Needs
        to be returned twice to save to the dictionary and then be able to be
        out of the function for use in next functions.

    """
    crop = []
    out = 0
    plot_image(tocrop, plotname)

    while out == 0:
        crop = tocrop
        decide = input("Do you want to run the crop for this video? Options are y/n: ")
        if decide == 'y':
            crop = auto_crop(tocrop, plotname)
            out = 1
            break
        elif decide == 'n':
            out = 1
        else:
            print("please type either y or n")
            out = 0

    return crop, crop

def auto_crop (tocrop, plotname):
    """
    Will request an imput from the user to determine how much of the image to
        crop off the sides, top, and bottom. Will produce a cropped image

    Inputs MUST be numerical. the program will fail if not numerical

    Parameters:
    -----------
    tocrop : array
        The raw tiff file that is stored in a dictionary and pulled from each
        key using a wrapper. Acts as the base image for the auto_crop

    plotname : string
        Name pulled from the orginal file name - is the chart title

    Returns:
    --------
    crop: array
        The array of the tiff file with the requested columns/rows removed

    """
    #intro constants
    TotalChange = 1
    left = 0
    right = 0
    top = 0
    bottom = 0

    #User inputs - plot will show between each iteration and will show updates with inputs
    while TotalChange != 0:
        crop = image_crop(tocrop, top, bottom, left, right)
        plot_image(crop, plotname)

        TotalChange = 0
        change = int(input("Enter the change you want for LEFT "))
        left = left + int(change)
        TotalChange = TotalChange + abs(change)

        change = int(input("Enter the change you want for RIGHT "))
        right = right + int(change)
        TotalChange = TotalChange + abs(change)

        change = int(input("Enter the change you want for TOP "))
        top = top + int(change)
        TotalChange = TotalChange + abs(change)

        change = int(input("Enter the change you want for BOTTOM "))
        bottom = bottom + int(change)
        TotalChange = TotalChange + abs(change)

    return crop


def inflection_points (crop, plotname, save_location):
    """
    This is a rewrap of the inflection point analysis function using the additive
        rows and columns to find the centriods. All function are the same, but
        the variable names have been changed to match the rest of the bulk
        wrapping functions

    IMPORTANT: This function assumes that the sample is being run on a 96 well
        plate. If this is not correct the number of detected wells will be off

    Parameters:
    -----------
    crop: array
        The array of the tiff file with the requested columns/rows removed

    Returns:
    --------
    inf_temp : list
        the inflection points of the wells in the video

    """

    img_eq = pa.image_eq(crop)
    column_sum, row_sum = pa.pixel_sum(img_eq)

    r_peaks, c_peaks = pa.peak_values(column_sum, row_sum, 12, 8, img_eq)
    sample_location = pa.locations(r_peaks, c_peaks, img_eq)

    temp, plate_temp = pa.pixel_intensity(sample_location, crop, 'Row', 'Column', 'plate_location')

    s_peaks, s_infl = ed.peak_detection(temp, plate_temp, 'Sample')
    p_peaks, p_infl = ed.peak_detection(temp, plate_temp, 'Plate')
    inf_temp = ed.inflection_point(temp, plate_temp, s_peaks, p_peaks)

    plot_profiles(temp, plate_temp, save_location)

    return inf_temp, inf_temp, temp, plate_temp

def bulk_crop (cv_file_names, location, d_all):
    """
    Wrapper for all of the bulk cropping functions. Wraps through all of the
        files in the inputed folder, asks for input if the user would like to
        crop the specific function, then asks for inputs for cropping then
        crops the specifed folder in the way requested. Then continues to loop
        through all of the files

    Parameters:
    -----------
    cv_file_names : list
        list of all of the file names in a specified folder, needs to be created
        before running the bulk wrapper

    location : string
        string containing the file location of the desired folder from the current
        location of the workbook

    d_all : dictonary
        Nested dictionary of all of the needed dictonary

    Returns:
    --------
    d_all : dictonary
        Nested dictionary of all of the needed dictonary, Should contain all the
        new additions from the functions
    """
    d_files, d_names, d_crop, d_inftemp, d_temp, d_plate = dict_unpack(d_all)

    for i,file in enumerate(cv_file_names):
        #file input
        d_files['%s' % i] = ed.input_file(location +str(file))

        # create names
        hold_name = cv_file_names[i]
        d_names['%s' % i] = hold_name[:-5]
        plotname = d_names[str(i)]
        keyname = str(i)

        if len(d_crop) == 0:
            tocrop = d_files['%s' % i]
        else:
            if keyname in d_crop:
                tocrop = d_crop['%s' % i]
            else:
                tocrop = d_files['%s' % i]

        #auto crops
        d_crop['%s' % i], crop = choose_crop(tocrop, plotname)

    d_all = dict_pack(d_files, d_names, d_crop, d_inftemp, d_temp, d_plate)

    return d_all

def bulk_analyze (cv_file_names, d_all):
    """
    Wrapper for all of the bulk analysis functions. Wraps through all of the
        files in the inputed folder. Runs analysis functions and then continues
        to loop through all of the files

    Parameters:
    -----------
    cv_file_names : list
        list of all of the file names in a specified folder, needs
        to be created before running the bulk wrapper

    d_all : dictonary
        Nested dictionary of all of the needed dictonary

    Returns:
    --------
    d_all : dictonary
        Nested dictionary of all of the needed dictonary, Should contain all the
        new additions from the functions

    all_inf : dataframe
        a dataframe with all of the sample wells and all of the frames. The
        columns will have the file name and the rows will have the well index

    """
    d_files, d_names, d_crop, d_inftemp, d_temp, d_plate = dict_unpack(d_all)

    all_inf = pd.DataFrame()

    for i, file in enumerate (cv_file_names):
        plotname = d_names[str(i)]
        keyname = str(i)

        save_location = 'temp_profiles/' + plotname
        if not os.path.exists(save_location):
            os.makedirs(save_location)

        crop = d_crop[keyname]
        #save inftemps
        d_inftemp['%s' % i], inf_temp, d_temp, d_plate = inflection_points(crop, plotname, save_location)
        #create df output
        all_inf[plotname] = inf_temp

    d_all = dict_pack(d_files, d_names, d_crop, d_inftemp, d_temp, d_plate)

    return d_all, all_inf

def bulk_process (cv_file_names, location, d_all):
    """
    Wrapper for all of the bulk functions. Runs the bulk cropper followed by the
        bulk analyzer.

    Parameters:
    -----------
    cv_file_names : list
        list of all of the file names in a specified folder, needs
        to be created before running the bulk wrapper

    location : string
        string containing the file location of the desired folder from the current
        location of the workbook

    d_all : dictonary
        Nested dictionary of all of the needed dictonary, should either be
        empty or from a previous run when an update is needed

    Returns:
    --------
    d_all : dictonary
        Nested dictionary of all of the needed dictonary, Should contain all the
        new additions from the functions

    all_inf : dataframe
        a dataframe with all of the sample wells and all of the frames. The
        columns will have the file name and the rows will have the well index

    """
    d_crop, d_names, d_all = bulk_crop(cv_file_names, location, d_all)

    d_inftemp, all_inf, d_all = bulk_analyze(cv_file_names, d_all)

    return d_all, all_inf



import tkinter as tk
from tkinter import filedialog


# Function: Exports data as a csv for user use
# Step: creates UI buttons to save, exports to user chosen place
# Input: dataframe of final data
# Output: no output - prompt to save the data to a csv
def export_csv(final_data):
    '''
    Exports the dataframe of the run into a csv file to be saved or
    modifed as desired

    Parameters
    -----------
    final_data : Dataframe
        A dataframe containing the final melting point
        data of all the samples.

    Returns
    --------
    Prompt to save the data to a csv.
    '''
    global df
    df = final_data

    root = tk.Tk()

    canvas1 = tk.Canvas(root, width=300, height=300, bg='lightgray',
                        relief='raised')
    canvas1.pack()

    root.attributes("-topmost", True)

    def exportCSV():
        global df

        export_file_path = filedialog.asksaveasfilename(defaultextension
                                                        ='.csv')
        df.to_csv(export_file_path, index=None, header=True)

    saveAsButton_CSV = tk.Button(text='Export CSV', command=exportCSV,
                                 bg='green', fg='white',
                                 font=('helvetica', 12, 'bold'))
    canvas1.create_window(150, 150, window=saveAsButton_CSV)

    root.mainloop()
    return

#  !! These functions/tests are still under development
# They relate specifically to the OPENTRONS pipetting robot,
# and will be refined for inner-lab use of the package/OPENTRONS combination.


# Function: adapts the pipetting order to the temperature reading order
# Step: collects all of the created lists, combines and labels the data
# Input: produced molfractions, and list of component names
# Output: reordered list of named compoents
# def molfrac_prep(DES_molfrac):
#     '''Adapts the pipetting order to the temperature reading order'''
#     #!! Assumes a 96 well plate in the opentrons pipetting
#         robot with 2 components
#     number = len(DES_molfrac[0])
#     array1 = np.zeros((8,12))
#     array2 = np.zeros((8,12))
#     comp1 = []
#     comp2 = []
#     ordered1 = []
#     ordered2 = []

#     for i in range(number):
#         if i == 0:
#             for row in DES_molfrac:
#                 hold = row[i]
#                 comp1.append(hold)
#         elif i == 1:
#             for row in DES_molfrac:
#                 hold = row[i]
#                 comp2.append(hold)
#         else:
#             pass
#     index = 0
#     for i in range(len(array1[0])):
#         for j in range(len(array1)):
#             array1[j, i] = comp1[index]
#             index = index +1
#     index= 0
#     for i in range(len(array2[0])):
#         for j in range(len(array2)):
#             array2[j, i] = comp2[index]
#             index = index +1
#     index=0
#     for i in range(len(array1)):
#         hold1 = array1[i]
#         ordered1.extend(hold1)
#     index=0
#     for i in range(len(array2)):
#         hold2 = array2[i]
#         ordered2.extend(hold2)

#     mol_data = pd.Dataframe()
#     mol_data['Component 1'] = ordered1
#     mol_data['Component 2'] = ordered2
#     return ordered1, ordered2


# Function: Creates the final dataframe of data
# Step: collects all of the created lists, combines and labels the data
# Input: all of the final data lists
# Output: dataframe of final info (mol inputs, melting temps, sample index)
# def create_dataframe_DES(all_melt, all_possible,
#                          samples, ordered1, ordered2):
#     '''Exports the dataframe of the run into a
#        csv file to be saved or modifed as desired'''
#     all_samples = []
#     for i in range(samples):
#         all_samples.append(i)

#     final_data = pd.DataFrame()

#     final_data['Sample Index'] = all_samples
#     final_data['Component 1'] = ordered1
#     final_data['Component 2'] = ordered2
#     final_data['Melting Temperature'] = all_melt
#     final_data['Other index possibilites'] = all_possible
#     return final_data

# Function: Creates the final dataframe of data NOT OPENTRONS DATA
# Step: collects all of the created lists, combines and labels the data
# Input: all of the final data lists
# Output: dataframe of final info (mol inputs, melting temps, sample index)
# def create_dataframe(all_melt, all_possible, samples):
#     '''Exports the dataframe of the run into a csv
#        file to be saved or modifed as desired'''
#     all_samples = []
#     for i in range(samples):
#         all_samples.append(i)

#     final_data = pd.DataFrame()

#     final_data['Sample Index'] = all_samples
#     final_data['Melting Temperature'] = all_melt
#     final_data['Other index possibilites'] = all_possible

#     return final_data



import os,sys
sys.path.append(os.path.dirname(os.path.abspath(__file__)))


import skimage
import numpy as np
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import edge_detection

from skimage import io
from skimage.draw import circle
from scipy.signal import find_peaks
from skimage.restoration import denoise_tv_chambolle
from irtemp import centikelvin_to_celsius

##########################################################################################################################################################################
##########################################################################################################################################################################
                                                #######Sample detection using peaks#########
##########################################################################################################################################################################
##########################################################################################################################################################################

# Image equalization
def image_eq(frames):
    '''
    Function to obtained an equalized image using all the frames
    in the video.
    
    Parameters
    -----------
    frames : List
        List of arrays of frames in the video.
    
    Returns
    --------
    img_eq: Array
        Equalized image  
    '''
    n_frames = len(frames)
    for II in range(n_frames):
        frame = frames[II]
        img_eq = (frame - np.amin(frame))/(np.amax(frame)-np.amin(frame))
        if II == 0:
            img_ave = img_eq
        else:
            img_ave = img_ave + img_eq
    img_average = img_ave/n_frames
    img_eq = (img_ave - np.amin(img_ave))/(np.amax(img_ave)-np.amin(img_ave))
    return img_eq

# Function to obtain sum of pixels over all the rows and columns
def pixel_sum(img_eq):
    ''' 
    Funtion to determine sum of pixels over all the rows and columns
    to obtain plots with peaks at the sample position in the array.
    
    Parameters
    -----------
        img_eq : Array
            Equalized image 
    
    Returns
    --------
        column_sum: List
            Sum of pixels over all the columns
        row_sum: List
            Sum of pixels over all the rows
            Also returns plots of column sum and row sum.
    '''
    
    # Denoising the image
    frame = denoise_tv_chambolle(img_eq)
    rows = frame.shape[0]
    columns = frame.shape[1]
    # Adding all the pixels in each column of the equalized image 
    column_sum = []        
    for i in range(0,columns):
        column_sum.append(sum(frame[:,i]))
    # Adding all the rows in each column of the equalized image 
    row_sum = []
    for j in range(0,rows):
        row_sum.append(sum(frame[j,:]))
    # To convert the troughs to peaks
    column_sum = [x * -1 for x in column_sum]
    # To convert the troughs to peaks
    row_sum = [x * -1 for x in row_sum]
    #Plotting the sum of pixel values in each column
    plt.plot(range(len(column_sum)),column_sum)
    plt.xlabel('Column index')
    plt.ylabel('Sum of pixel values over columns')
    plt.title('Sum of pixel values over columns against column index')
    plt.show()
    #Plotting the sum of pixel values in each row
    plt.plot(range(len(row_sum)),row_sum)
    plt.xlabel('Row index')
    plt.ylabel('Sum of pixel values over rows')
    plt.title('Sum of pixel values over rows against row index')
    plt.show()
    return column_sum,row_sum

# To determine the peak values in the row and column sum and thus sample
# location.
def peak_values(column_sum,row_sum,n_columns,n_rows,image, freeze_heat):
    '''
    Function to find peaks from the column_sum and row_sum arrays
    and return a dataframe with sample locations.
    Use this when the wells are at a lower temperature when compared to the 
    plate in 'img_eq'
    
    Parameters
    -----------
        column_sum: List
            Sum of pixel values over all the columns in the
            image array.
        row_sum: List
            Sum of pixel values over all the rows in the
            image array.
        n_columns: Int
            Number of columns of samples in the image
        n_rows: Int
            Number of rows of samples in the image.
        image: Array
            Image to be processed
        
    Returns
    -------- 
        sample_location: Dataframe
            A dataframe containing sample and plate locations and a plot with locations
            superimposed on the image to be processed.
    '''
    if freeze_heat == True:
        # Converting the troughs to peaks
        column_sum = [x*-1 for x in column_sum]
        row_sum = [x*-1 for x in row_sum]
    # Finding peaks in the column sum array
    all_column_peaks = find_peaks(column_sum, height = (None, 1000), distance=7)
    # Getting column peaks as a list
    column_indices = list(all_column_peaks[0])
    # Getting column peak_heights
    column_heights = list(all_column_peaks[1]['peak_heights'])
    # Picking the highest peak values equal to the number of sample columns
    column_peak_indices = []
    for i in range(n_columns):
        column_peaks = column_heights.index(max(column_heights))
        column_peak_indices.append(column_indices[column_peaks])
        pop_index = column_heights.index(max(column_heights))
        column_heights.pop(pop_index)
        column_indices.pop(pop_index)
    # Sorting column indices in ascending order
    column_peak_indices.sort()
    # Finding peaks in the row sum array
    all_row_peaks = find_peaks(row_sum, height = (None, 1000), distance=7)
    # Getting row peaks as a list
    row_indices = list(all_row_peaks[0])
    # Getting peak heights
    row_heights = list(all_row_peaks[1]['peak_heights'])
    # Picking the highest peak values equal to the number of sample rows
    row_peak_indices = []
    for i in range(n_rows):
        row_peaks = row_heights.index(max(row_heights))
        row_peak_indices.append(row_indices[row_peaks])
        pop_index = row_heights.index(max(row_heights))
        row_heights.pop(pop_index)
        row_indices.pop(pop_index)
    # Sorting row indices in ascending order
    row_peak_indices.sort()
    return row_peak_indices, column_peak_indices
    

def locations(row_peak_indices, column_peak_indices, image):
    '''
    Function to get location of all the samples(row and column) and their
    respective plate locations. (Same column but different rows)
    
    Parameters
    -----------
    row_peak_indices : List
        List containing the location of all the sample rows
    column_peak_indices : List
        List containing the location of all the sample columns.
    
    Returns
    --------
    sample_location : Dataframe
        Dataframe containing sample location and plate location.
    '''
    # Appending row and column peak values to arrays to get plate location.
    row = []
    column = []
    plate_location = []
    n_columns = len(column_peak_indices)
    n_rows = len(row_peak_indices)
    for i in range(0,n_columns):
        for j in range(0,n_rows):
            row.append(row_peak_indices[j])
            column.append(column_peak_indices[i])
            if j == 0:
                # Calculating plate location for each well based on the well location.
                plate_location.append(int((row[j]-0)/2))
            else:
                plate_location.append(int((row[j] + row[j-1])/2))
    # Dataframe containing the well location(row and column)
    sample_location = pd.DataFrame(list(zip(row, column, plate_location)),columns =['Row', 'Column', 'plate_location'])
    plt.imshow(image)
    plt.scatter(sample_location['Column'],sample_location['Row'],s=4,color='Red')
    plt.scatter(sample_location['Column'],sample_location['plate_location'],s=4,color='Purple')
    plt.title('Sample and plate location at which the temperature profile is monitored')
    plt.show()
    return sample_location

# To determine the sample and plate temperature using peak locations.
def pixel_intensity(sample_location, frames, x_name, y_name, plate_name):
    ''' 
    Function to find pixel intensity at all sample locations
    and plate locations in each frame.

    Parameters
    -----------
    sample_location : Dataframe
        A dataframe containing sample and plate locations.
    frames : Array
        An array of arrays containing all the frames of a video.
    x_name : String
        Name of the column in sample_location containing the row values of the samples.
    y_name : String
        Name of the column in sample_location containing the column values of the samples.
    plate_name : String
        Name of the column in sample_location containing the column values of the
        plate location.

    Returns
    --------
    temp : List
        Temperature of all the samples in every frame of the video.
    plate_temp : List
        Temperature of the plate next to every sample in every
        frame of the video.
    '''
    temp = []
    plate_temp = []
    # Row values of the peaks
    x = sample_location[x_name]
    # Column values of the peaks
    y = sample_location[y_name]
    # Row value of the plate location
    p = sample_location[plate_name]
    for i in range(len(sample_location)):
        temp_well = []
        plate_well_temp = []
        for frame in frames:
            rr, cc = circle(x[i], y[i], radius = 1)
            sample_intensity = np.mean(frame[rr,cc])
            temp_well.append(centikelvin_to_celsius(frame[x[i]][y[i]]))
            plate_well_temp.append(centikelvin_to_celsius(frame[p[i]][y[i]]))
        temp.append(temp_well)
        plate_temp.append(plate_well_temp)
    return temp, plate_temp


##### Wrapping Function ######
def pixel_temp(frames,n_columns,n_rows,freeze_heat):
    ''' 
    Function to determine the temperature of the samples and plate locations by analysing 
    pixel values and finding peaks.
    
    Parameters
    -----------
    frames: Array
        The frames of a video to be analysed.
    n_columns: Int
        Number of columns of samples in the image
    n_rows: Int
        Number of rows of samples in the image.
    
    Returns
    --------
    m_df : Dataframe
        A dataframe containing row and column coordinates of each sample 
        and its respective inflection point obtained. 
    '''
    # flip_frames = edge_detection.flip_frame(frames)
    #Function to obtained an equalized image using all the frames
    #in the video.
    img_eq = image_eq(frames)
    #Funtion to determine sum of pixels over all the rows and columns
    #to obtain plots with peaks at the sample position in the array.
    column_sum,row_sum = pixel_sum(img_eq)
    # Function to find peaks from the column_sum and row_sum arrays
    r_peaks, c_peaks = peak_values(column_sum,row_sum,n_columns,n_rows,img_eq,freeze_heat)
    # Function to return a dataframe with sample locations and respective plate locations.
    sample_location = locations(r_peaks, c_peaks, img_eq)
    # Function to find pixel intensity at all sample locations
    # and plate locations in each frame.
    temp, plate_temp = pixel_intensity(sample_location, frames, x_name = 'Row', y_name = 'Column', plate_name = 'plate_location')
    #Function to obtain the peaks in sample temperature profile
    s_peaks, s_infl = edge_detection.peak_detection(temp, plate_temp, 'Sample')
    # # Function to obtain the peaks in plate location temperature profile
    # p_peaks, p_infl = edge_detection.peak_detection(temp, plate_temp, 'Plate')
    # # Function to obtain the inflection point(melting point) from the temperature profile.
    # inf_temp = inflection_point(temp, plate_temp, s_peaks, p_peaks)
    # Dataframe with sample location (row and column coordinates) and respective inflection point.
    m_df = pd.DataFrame({'Row':sample_location.Row,'Column':sample_location.Column,'Melting point':np.asarray(s_infl)[:,0]})
    return m_df




import os
import sys

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

import numpy as np
import pandas as pd
import random
import h5py

from skimage import io
from skimage import feature
from skimage.draw import circle
from scipy.ndimage.morphology import binary_fill_holes
from skimage.measure import label
from skimage.measure import regionprops
from skimage.morphology import remove_small_objects
from scipy.signal import find_peaks
from scipy.interpolate import interp1d
from scipy.signal import filtfilt
from scipy.interpolate import BSpline
from irtemp import centikelvin_to_celsius


# Function to load the input file
def input_file(file_name):
    '''
    To load the imput file as an array.

    Parameters
    -----------
    file_name : String
        Name of the Tiff or HDF5 file to be loaded 
        as it is saved on the disk.
        Provide file path if it is not in the same directory as
        the jupyter notebook.

    Returns
    --------
    frames : Array
        In case of a video, returns an array for each frame
        in the video.
        In case of an image, return an array.
    '''
    file_type = file_name[-4:]
    if file_type == 'HDF5':
        file = h5py.File(file_name, 'r')
        frames = []
        for i in range(1, len(file.keys())+1):
            frames.append(file['image'+str(i)])
    elif file_type == 'tiff':
        frames = io.imread(file_name)
    return frames


# Function to flip the frames horizontally and vertically to correct
# for the mirroring during recording.
def flip_frame(frames):
    '''
    To flip all the loaded frames horizontally and vertically
    to correct for the mirroring during recording.

    Parameters
    -----------
    frames : Array
        An array containing an array for each frame
        in the video or just a single array in case of an image.

    Returns
    --------
    flip_frames : Array
        Flipped frames that can be processed to get temperature data.
    '''
    flip_frames = []
    for frame in frames:
        f_frame = np.fliplr(frame)
        flip_frames.append(np.flipud(f_frame))
    return flip_frames


# Function to detect edges, fill and label the samples.
def edge_detection(frames, n_samples):
    '''
    To detect the edges of the wells, fill and label them to
    determine their centroids.

    Parameters
    -----------
    frames : Array
        The frames to be processed and determine the
        sample temperature from.
    n_samples : Int
        The number of samples in the input video.

    Returns
    --------
    labeled_samples : Array
        All the samples in the frame are labeled
        so that they can be used as props to get pixel data.
    '''
    for size in range(15, 9, -1):
        for thres in range(1500, 900, -100):
            edges = feature.canny(frames[0]/thres)
            filled_samples = binary_fill_holes(edges)
            cl_samples = remove_small_objects(filled_samples, min_size=size)
            labeled_samples = label(cl_samples)
            props = regionprops(labeled_samples, intensity_image=frames[0])
            if len(props) == n_samples:
                break
#             if thres == 1000 and len(props) != n_samples:
#                 print('Not all the samples are being recognized with
#                 the set threshold range for size ',size)
        if len(props) == n_samples:
            break
    if size == 10 and thres == 1000 and len(props) != n_samples:
        print('Not all the samples are being recognized with the set \
            minimum size and threshold range')
    return labeled_samples


# Function to determine centroids of all the samples
def regprop(labeled_samples, frames, n_rows, n_columns):
    '''
    Determines the area and centroid of all samples.

    Parameters
    -----------
    labeled_samples: Array
        An array with labeled samples.

    frames : Array
        Original intensity image to determine
        the intensity at sample centroids.
    n_rows: Int
        Number of rows of sample
    n_columns: Int
        Number of columns of sample

    Returns
    --------
    regprops: Dict
        A dictionary of dataframes with information about samples in every
        frame of the video.
    '''
    regprops = {}
    n_samples = n_rows * n_columns
    unique_index = random.sample(range(100), n_samples)
    for i in range(len(frames)):
        props = regionprops(labeled_samples, intensity_image=frames[i])
        # Initializing arrays for all sample properties obtained from regprops.
        row = np.zeros(len(props)).astype(int)
        column = np.zeros(len(props)).astype(int)
        area = np.zeros(len(props))
        radius = np.zeros(len(props))
        perim = np.zeros(len(props))
        intensity = np.zeros(len(props), dtype=np.float64)
        plate = np.zeros(len(props), dtype=np.float64)
        plate_coord = np.zeros(len(props))

        c = 0
        for prop in props:
            row[c] = int(prop.centroid[0])
            column[c] = int(prop.centroid[1])
            # print(y[c])
            area[c] = prop.area
            perim[c] = prop.perimeter
            radius[c] = prop.equivalent_diameter/2
            rr, cc = circle(row[c], column[c], radius = radius[c]/3)
            intensity[c] = np.mean(frames[i][rr,cc])
            plate[c] = frames[i][row[c]][column[c]+int(radius[c])+3]
            plate_coord[c] = column[c]+radius[c]+3
            c = c + 1
        regprops[i] = pd.DataFrame({'Row': row, 'Column': column,
                                    'Plate_temp(cK)': plate,
                                    'Radius': radius,
                                    'Plate_coord': plate_coord,
                                    'Area': area, 'Perim': perim,
                                    'Sample_temp(cK)': intensity,
                                    'unique_index': unique_index},
                                   dtype=np.float64)
        if len(regprops[i]) != n_samples:
            print('Wrong number of samples are being detected in frame %d' % i)
        regprops[i].sort_values(['Column', 'Row'], inplace=True)
    return regprops


def sort_regprops(regprops, n_columns, n_rows):
    '''
    Function to sort the regprops to match the order in which the samples
    are pipetted.

    Parameters
    ------------
    regprops : Dict
        A dictionary of dataframes containing information about the sample.
    n_columns : Int
        Number of columns of samples
    n_rows : Int
        Number of rows of samples

    Returns
    --------
    sorted_regprops : Dict
        A dictionary of dataframe with information about samples in every
        frame of the video. The order of the samples is sorted from
        top to bottom and from left to right.
    '''
    sorted_regprops = {}
    # n_samples = n_columns * n_rows
    # After sorting the dataframe according by columns in ascending order.
    sorted_rows = []
    # Sorting the dataframe according to the row coordinate in each column.
    # The samples are pipetted out top to bottom from left to right.
    # The order of the samples in the dataframe
    # should match the order of pipetting.
    for j in range(0, n_columns):
        df = regprops[0][j*n_rows:(j+1)*n_rows].sort_values(['Row'])
        sorted_rows.append(df)
    regprops[0] = pd.concat(sorted_rows)
    # Creating an index to be used for reordering all the dataframes.
    # The unique index is the sum of row and column coordinates.
    reorder_index = regprops[0].unique_index
    for k in range(0, len(regprops)):
        regprops[k].set_index('unique_index', inplace=True)
        sorted_regprops[k] = regprops[k].reindex(reorder_index)
    return sorted_regprops


# Function to obtain temperature of samples and plate temp
def sample_temp(sorted_regprops, frames):
    '''
    Function to concatenate all the obtained temperature data
    from the pixel values into lists.

    Parameters
    ----------
    sorted_regprops : Dict
        The dictionary of sorted dataframes containing temperature data.
    frames : Array
        The array of frames to be processed to obtain temperature data.

    Returns
    -------
    temp : List
        Temperature of all the samples in every frame of the video.
    plate_temp : List
        Temperature of the plate next to every sample in every
        frame of the video.
    '''
    temp = []
    plate_temp = []
    for j in range(len(sorted_regprops[1])):
        temp_well = []
        plate_well_temp = []
        for i in range(len(frames)):
            temp_well.append(centikelvin_to_celsius
                             (list(sorted_regprops[i]['Sample_temp(cK)'])[j]))
            plate_well_temp.append(centikelvin_to_celsius(list
                                   (sorted_regprops[i]['Plate_temp(cK)'])[j]))
        temp.append(temp_well)
        plate_temp.append(plate_well_temp)
    return temp, plate_temp


# # Function to obtain melting point by extracting the inflection point
# def peak_detection(sample_temp, plate_temp, material):
#     '''
#     Function to determine inflection point in the sample temperature
#     profile(melting point)

#     Parameters
#     -----------
#     sample_temp : List
#         Temperature of all the samples in every frame of the video.
#     plate_temp : List
#         Temperature profiles of all the plate locations
#     material : String
#         Can be 'Plate' or 'Sample'

#     Returns
#     --------
#     peaks : List
#         List of two highest peak(inflection points) indices in the
#         given temperature profiles.
#     infl : List
#         List of temperature at inflection points for
#         given temperature profiles.

#     '''
#     infl = []
#     peak_indices = []
#     for i in range(len(sample_temp)):
#         frames = np.linspace(1,len(sample_temp[i]),len(sample_temp[i]))
#         # Fitting a spline to the temperature profile of the samples.
#         if material == 'Plate':
#             bspl = BSpline(frames,plate_temp[i],k=3)
#             # Stacking x and y to calculate gradient.
#             gradient_array = np.column_stack((frames,bspl(frames)))
#         else:
#             f = interp1d(plate_temp[i], sample_temp[i],bounds_error=False)
#             gradient_array = np.column_stack((plate_temp[i],f(plate_temp[i])))
#         # Calculating gradient
#         gradient = np.gradient(gradient_array,axis=0)
#         # Calculating derivative
#         derivative = gradient[:,1]/gradient[:,0]
#         # Finding peaks in the derivative plot.
#         peaks, properties = find_peaks(derivative, height=0)
#         # Peak heights
#         peak_heights = properties['peak_heights']
#         a = list(peak_heights)
#         max_height1 = np.max(a)
#         a.remove(max_height1)
#         max_height2 = np.max(a)
#         # Appending the index of the two highest peaks to lists.
#         inf_index1 = list(peak_heights).index(max_height1)
#         inf_index2 = list(peak_heights).index(max_height2)
#         # Appending the frame number in which these peaks occur to a list
#         peak_indices.append([peaks[inf_index1],peaks[inf_index2]])
#         # Appending the temperature at the peaks.
#         if material == 'Plate':
#             infl.append([plate_temp[i][peaks[inf_index1]],
#                         plate_temp[i][peaks[inf_index2]]]) 
#         else:
#             infl.append([sample_temp[i][peaks[inf_index1]],
#                         sample_temp[i][peaks[inf_index2]]])
#     return peak_indices, infl

# Function to obtain melting point by extracting the inflection point
def peak_detection(sample_temp, plate_temp, material):
    '''
    Function to determine inflection point in the sample temperature
    profile(melting point)

    Parameters
    -----------
    sample_temp : List
        Temperature of all the samples in every frame of the video.
    plate_temp : List
        Temperature profiles of all the plate locations
    material : String
        Can be 'Plate' or 'Sample'

    Returns
    --------
    peaks : List
        List of two highest peak(inflection points) indices in the
        given temperature profiles.
    infl : List
        List of temperature at inflection points for
        given temperature profiles.

    '''
    infl = []
    peak_indices = []
    for i in range(len(sample_temp)):
        # Fitting a spline to the temperature profile of the samples.
#         if material == 'Plate':
#             bspl = BSpline(frames,plate_temp[i],k=3)
#             # Stacking x and y to calculate gradient.
#             gradient_array = np.column_stack((frames,bspl(frames)))
#         else:
        f = interp1d(plate_temp[i], sample_temp[i],bounds_error=False)
        x = np.linspace(min(plate_temp[i]), max(plate_temp[i]), len(plate_temp[i]))
        y = f(x)
        n = 25  # the larger n is, the smoother curve will be
        b = [1.0 / n] * n
        a = 1
        yy = filtfilt(b,a,y)
        gradient_array = np.column_stack((x,yy))
        # Calculating gradient
        first_gradient = np.gradient(gradient_array,axis=0)
        # Calculating derivative
        derivative = first_gradient[:,1]/first_gradient[:,0]
        # Finding peaks in the derivative plot.
        peaks, properties = find_peaks(derivative, height=0)
        # Peak heights
        peak_heights = properties['peak_heights']
        a = list(peak_heights)
        max_height1 = np.max(a)
        a.remove(max_height1)
        max_height2 = np.max(a)
        # Appending the index of the two highest peaks to lists.
        inf_index1 = list(peak_heights).index(max_height1)
        inf_index2 = list(peak_heights).index(max_height2)
        # Appending the frame number in which these peaks occur to a list
        peak_indices.append([peaks[inf_index1],peaks[inf_index2]])
        # Appending the temperature at the peaks.
        if material == 'Plate':
            infl.append([x[peak_indices[i][0]],
                        x[peak_indices[i][1]]]) 
        else:
            infl.append([yy[peak_indices[i][0]],
                        yy[peak_indices[i][1]]])
    return peak_indices, infl


# def inflection_point(s_temp, p_temp, s_peaks, p_peaks):
#     '''
#     Function to get the inflection point(melting point) for each sample.

#     Parameters
#     -----------
#     s_temp : List
#         Sample temperature profiles
#     p_temp : List
#         Plate location temperature profiles
#     s_peaks : List
#         List of two highest peak(inflection points) indices in the
#         temperature profile of the samples.
#     p_peaks : List
#         List of two highest peak(inflection points) indices in the
#         temperature profile of the plate.

#     Returns
#     --------
#     inf_temp : List
#         List of temperature at inflection points for each sample

#     '''
#     inf_peak = []
#     inf_temp = []
#     for i, peaks in enumerate(s_peaks):
#         for peak in peaks:
#             # Making sure the peak is present only in the sample temp profile
#             if abs(peak - p_peaks[i][0]) >= 3:
#                 inf_peak.append(peak)
#                 break
#             else:
#                 pass
#     # Appending the temperature of the sample at the inflection point
#     for i, temp in enumerate(s_temp):
#         inf_temp.append(temp[inf_peak[i]])
#     return inf_temp


# Wrapping functions
# Wrapping function to get the inflection point
def inflection_temp(frames, n_rows, n_columns):
    '''
    Function to obtain sample temperature and plate temperature
    in every frame of the video using edge detection.

    Parameters
    -----------
    frames : List
        An list containing an array for each frame
        in the video or just a single array in case of an image.
    n_rows: List
        Number of rows of sample
    n_columns: List
        Number of columns of sample

    Returns
    --------
    frames : Array
        An array of images which are flipped to correct the
        rotation caused by the IR camera
    regprops : Dict
        A dictionary of dataframes containing temperature data.
    s_temp : List
        A list containing a list a temperatures for each sample
        in every frame of the video.
    plate_temp : List
        A list containing a list a temperatures for each plate
        location in every frame of the video.
    inf_temp : List
        A list containing melting point of all
        the samples obtained by the plot.
    m_df : Dataframe
        A dataframe containing row and column coordinates of each sample
        and its respective inflection point obtained.

    '''

    # Determining the number of samples
    n_samples = n_columns * n_rows
    # Use the function 'flip_frame' to flip the frames horizontally
    # and vertically to correct for the mirroring during recording
    # flip_frames = flip_frame(frames)
    # Use the function 'edge_detection' to detect edges, fill and
    # label the samples.
    labeled_samples = edge_detection(frames, n_samples)
    # Use the function 'regprop' to determine centroids of all the samples
    regprops = regprop(labeled_samples, frames, n_rows, n_columns)
    # Use the function 'sort_regprops' to sort the dataframes in regprops
    sorted_regprops = sort_regprops(regprops, n_columns, n_rows)
    # Use the function 'sample_temp' to obtain temperature of samples
    # and plate temp
    s_temp, p_temp = sample_temp(sorted_regprops, frames)
    # Use the function 'sample_peaks' to determine the inflections points
    # and temperatures in sample temperature profiles
    s_peaks, s_infl = peak_detection(s_temp, p_temp, 'Sample')
    # # Use the function 'plate_peaks' to determine the inflections
    # # in plate temperature profiles
    # p_peaks, p_infl = peak_detection(s_temp, p_temp, 'Plate')
    # # Use the function 'infection_point' to obtain melting point of samples
    # inf_temp = inflection_point(s_temp, p_temp, s_peaks, p_peaks)
    # Creating a dataframe with row and column coordinates
    # of sample centroid and its melting temperature (Inflection point).
    m_df = pd.DataFrame({'Row': regprops[0].Row, 'Column': regprops[0].Column,
                        'Melting point': np.asarray(s_infl)[:,0]})
    return sorted_regprops, s_temp, p_temp, s_infl, m_df



import os,sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# from musicalrobot import DES

from sklearn.cluster import KMeans
import random
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.optimize import fsolve

# Function: General Des generator - any number of components will work
# Step: Generate a large amount of possibilities, narrow down within constraints
# Input: min and max mol fraction desired, desired output samples, with trial estimate
# Output: list of mol fractions for each component
def des_generator(min_comps, max_comps, samples, trials):
    '''Generates list of mol fractions with any amount of samples'''
    #The minimum mole fractions of each component.
    lower_bounds = np.array(min_comps)
    upper_bounds = np.array(max_comps)

    #Generating random DES compositions within the design contraints, mole fractions of each composition must = 1.
    DES_trials = np.random.rand(trials*20,2)
    DES_trials = DES_trials*(upper_bounds-lower_bounds)+lower_bounds

    #Adding mole fractions of each component in the random trial.
    mole_sum = np.sum(DES_trials, axis=1)
    DES_samples = DES_trials/mole_sum[:,None]

    #This normalization may still lead to compositions that do not satisfy the constraint
    upper_check = DES_samples>upper_bounds
    lower_check = DES_samples<lower_bounds
    combined_check = np.append(upper_check, lower_check, axis=1)
    SafeList = np.any(combined_check, axis=1)
    DeleteList = ~SafeList
    Feasible_DES_samples = DES_samples[DeleteList,:]
    print(" "+str(len(Feasible_DES_samples))+" feasible DES samples generated, clustered into "+str(samples)+" samples")

    #Apply K-means clustering to DES samples
    kmeans = KMeans(n_clusters=samples, random_state=0).fit(Feasible_DES_samples)
    DES_molfrac = kmeans.cluster_centers_
    return DES_molfrac


# Function: Converts mol fractions to volumes for pipetting and reference
# Step: create system of equations, solve system to determine the volume of each
# Input: DES mol fractions, desired vol, stock solutions
# Output: list of volumes of each component for the desired volume
def mol_to_vol(DES_molfrac, stock, volume):
    '''Converts mol fractions to volumes depending on desired volume and stocks'''
    # pre-processing
    number = len(stock)
    samples = len(DES_molfrac)
    size = (samples, number)
    final_vol = np.zeros(size)


    count = 0
    for row in DES_molfrac:
        def f(x):
            for i in range(number):
                total = []
                place = stock[i]*x[i]
                total.append(place)
            total = sum(total)
            y = np.zeros(np.size(x))
            y[0] = x[0] + x[1] - volume
            for i in range(number-1):
                y[i+1] = (((stock[i])*x[i])/(total)) - row[i]
            return y
        x0 = np.array([100.0, 100.0, 100.0])
        x = fsolve(f, x0)
        for i in range(number):
            final_vol[count, i] = x[i]
        count = count + 1
    return final_vol


# Function: Convert list of volumes into a list that opentrons can use
# Step: separate lists
# Input: final_vol
# Output: open_vol - usuable by opentrons machine
def open_vol(final_vol):
    '''Converts the array of volumes with comp volumes to two lists of separate volumes'''
    number = len(final_vol[0])
    open_vol = []

    for i in range(number):
        string = "comp" + str(i)
        string = []
        for row in final_vol:
            hold = row[i]
            string.append(hold)
        open_vol.append(string)
    return open_vol



# SET UP FILE:
# in order to run in notebooks as an import musicalrobot
# 1. Git clone the repository to a local computer
# 2. go to the outermost musical-robot folder
# 3. use "pip install . "
# 4. import packages into a jupyter notebook using "from musicalrobot import xxxxxx"

from setuptools import setup

setup(name = 'musicalrobot',
    version = '0.973',
    packages = ['musicalrobot'],
    url = 'https://github.com/pozzocapstone/musical-robot',
    license = 'MIT',
    author = 'Shrilakshmi Bonageri, Jaime Rodriguez, Sage Scheiwiller',
    description= 'A package for high-throughput measurement of deep eutectic solvents’ melting point using IR bolometry',
    description_content_type = 'text/markdown; charset=UTF-8; variant=GFM',
    short_description = 'Melting temperature determination using IR bolometry',
    short_description_content_type = 'text/markdown',
    long_description = open('README.md','r').read(),
    long_description_content_type = 'text/markdown; charset=UTF-8; variant=GFM',
    zip_safe = False,
)



import os
import sys

sys.path.append(os.path.dirname(os.path.abspath(__file__)))


# Function 1: Converts the raw centikelvin reading to Celcius
# Step: convert using given formula for centikelvin to celcius
# Input: centikelvin reading
# Output: float value in celcius
def centikelvin_to_celsius(temp):
    '''
    Converts given centikelvin value to Celsius

    Parameters
    -----------
    temp : Float
        The value of the temperature to be converted in centikelvin

    Returns
    --------
    cels : Float
        The converted value of the temperature in degree celcius
    '''
    cels = (temp - 27315)/100
    return cels


# Function: Converts raw centikelvin reading to fahrenheit
# Step:Use function (1) to convert to cels, use equation to convert to fahr
# Input: centikelvin reading
# Output: float value in fahrenheit
def to_fahrenheit(temp):
    '''
    Converts given centikelvin value to Celsius

    Parameters
    -----------
    temp : Float
        The value of the temperature to be converted in centikelvin

    Returns
    --------
    fahr : Float
        The converted value of the temperature in degree fahrenheit
    '''
    cels = centikelvin_to_celsius(temp)
    fahr = cels * 9 / 5 + 32
    return fahr


# Function: Covnerts raw centikelvin value to both fahrenheit and celcius
# Step: Use function (1) to convert to cels, use equation to convert to fahr
# Input: centikelvin reading
# Output: float values in celcius and fahrenheit
def to_temperature(temp):
    '''
    Converts given centikelvin value to both fahrenheit and celcius

    Parameters
    -----------
    temp : Float
        The value of the temperature to be converted in centikelvin

    Returns
    --------
    cels : Float
        The converted value of the temperature in degree celcius
    fahr : Float
        The converted value of the temperature in degree fahrenheit
    '''
    cels = centikelvin_to_celsius(temp)
    fahr = cels * 9 / 5 + 32
    return cels, fahr


# # Function: Determines the slope between
# # temperature points with some smoothing
# # Step: append all slopes and index points to variables,
# # using given jump distance
# # Input: jump, the sample temp data set for given sample
# # Output: slope for each requested jump, index for each requested jump
# def slope_gen(sample_temp, plate_temp, jump):
#    '''Determines the slope between temperature points with some smoothing'''
#     # reccomended jump higher than 15
#     y = sample_temp
#     x = plate_temp
#     all_slope = []
#     all_index = []
#     index = 0

#     while index < len(y):
#         if index >= len(y) - jump:
#             break

#         if x[index]-x[index+jump] == 0:
#             jump = jump + 1
#             index = 0
#         else:
#             pass

#         slope = (y[index]-y[index + jump])/(x[index]-x[index + jump])

#         all_slope.append(slope)
#         all_index.append(index)
#         index = index + jump

#     return all_slope, all_index

# # Function: Smoothes the results generated in slope
# # gen to coax the max dense zones out
# # Step: average the slope and index over a range of values
# # to produce a more averaged graph
# # Input: reults of slope_gen (all_slopes and all indexes in the set),
# # smoothing factor
# # Output: the resulting smoothed index and the slopes
# def smoothing_slope(all_slope, all_index, factor):
#     '''Smoothes the results generated in slope gen to
#        coax the max dense zones out'''
#     #smoothing factor allows for more or less smoothing, reccomend around 5
#     smooth_slope = []
#     smooth_index = []

#     for i in range(len(all_slope)):
#         smooth_slope.append(sum(all_slope[i:i+factor])/factor)
#         smooth_index.append(sum(all_index[i:i+factor])/factor)
#     return smooth_slope, smooth_index

# # Function: Produces temperature value of the most possible inflection point
# # Step: set constraints on the ideal slope, generate list of possible,
# # average
# # Input: smooth_slope and smooth_index
# # Output: temperature of most likely, list of other close/possible points
# def inflection_temp(smooth_slope, smooth_index, sample_temp):
#     '''Produces temperature value of the most possible inflection point'''
#     yposs =[]
#     index_poss = []

#     for i in range(len(smooth_slope)):
#         upbound = 2*np.std(smooth_slope) + np.mean(smooth_slope)
#         downbound = np.mean(smooth_slope) - 2 * np.std(smooth_slope)
#     if smooth_slope[i] < downbound:
#         yposs.append(smooth_slope[i])
#         index_poss.append(smooth_index[i])
#     elif smooth_slope[i] > upbound:
#         yposs.append(smooth_slope[i])
#         index_poss.append(smooth_index[i])
#     else:
#         pass
#     if len(index_poss) == 0:
#         print('no inflection point found')
#         melt_temp = False
#         return melt_temp, index_poss
#     else:
#         pass
#     check = int(np.mean(index_poss))
#     melt_temp = sample_temp[check]
#     return melt_temp, index_poss


# # Function:Wrapping function for single sample
# # Step: combines all of the following inflection point functions into one
# # Input: sample temperatures, desired intial jump, smoothing factor
# # Output: melting temperature and other possible values if
# # determined to be incorrect
# def melting_temperature(sample_temp, plate_temp, jump, factor):
#     '''Wraps all of the inflection point functions to a single one'''
#     all_slope, all_index = slope_gen(sample_temp, plate_temp, jump)
#     smooth_slope, smooth_index = smoothing_slope(all_slope,
#                                                  all_index, factor)
#     melt_temp, index_poss = inflection_temp(smooth_slope,
#                                             smooth_index, sample_temp)
#     return melt_temp, index_poss


# # Function: Looping through all samples to determine all melting temperatures
# # Step: loop through the different samples running the wrapped melting
# # temp function for each one
# # Input: all sample temperatures for all samples
# # Output: all sample melting temperature,
# # all possible indexes of other melting temps
# def all_melting(all_sample_temp, plate_temp, jump, factor):
#     '''Looping through all samples to determine all melting temperatures'''
#     all_melt = []
#     all_possible = []

#     for i in range(len(all_sample_temp)):
#         sample_temp = all_sample_temp[i]
#         hold_melt, hold_possible = melting_temperature(sample_temp,
#                                                        plate_temp,
#                                                        jump, factor)

#         all_melt.append(hold_melt)
#         all_possible.append(hold_possible)
#     return all_melt, all_possible
